Scriptname REQ_PassiveArmorTraining extends REQ_PlayerAlias
{increase armor skills by moving in armor (or clothes) and prevents armor changing in combat, also evaluates the mass actor value}

Int[] Property contributions Auto
{all biped slots which contribute to skillgain}
Float[] Property Skillgain Auto
{skillgain per interval for given contributing slot while running, sprinting is *2}

Float Property UpdateInterval Auto
{time between consecutive skill increment checks}
Message Property NoCombatChanges Auto
{Notification to player, informing him that he must not change armor in combat}
Armor[] Property Exceptions Auto
{armors which can be changed in combat to prevent problems with transformations}

GlobalVariable Property ArmorDebug Auto
{enable debug notifications for armor increments}
GlobalVariable Property CombatToggle Auto
{toggling of the feature}

Float gain_light
Float gain_heavy
Armor[] slots
Formlist Property worn Auto
{an empty formlist, contains the currently worn equipment at runtime to evaluate equipment mass}
Float mass = 0.0
Spell Property MassEffect Auto
Perk Property MassPerk Auto

Bool working = False

Event ScriptInit(string eventName, string strArg, float numArg, Form sender)
	slots = new Armor[32]
	AdjustGains()
	player.RemoveSpell(MassEffect)
	While !Player.HasPerk(MassPerk)
		Utility.Wait(0.5)
	EndWhile
	player.AddSpell(MassEffect,False)
	player.modAV("CarryWeight",-0.1)
	player.modAV("CarryWeight",0.1)
	RegisterForUpdate(UpdateInterval)
	REQ_AutoUpdater UpdateControl = GetOwningQuest().GetAlias(0) as REQ_AutoUpdater
	Int index = UpdateControl.Scripts.find(self)
	UpdateControl.ShutdownReady[index] = False
EndEvent

Event ScriptShutDown(string eventName, string strArg, float numArg, Form sender)
	player.RemoveSpell(MassEffect)
	player.modav("mass", -mass)
	UnregisterForUpdate()
	REQ_AutoUpdater UpdateControl = GetOwningQuest().GetAlias(0) as REQ_AutoUpdater
	Int index = UpdateControl.Scripts.find(self)
	UpdateControl.ShutdownReady[index] = True
EndEvent

Event OnUpdate()
	If Player.IsRunning()
		If (gain_light > 0)
			Game.AdvanceSkill("Lightarmor" , gain_light)
		EndIf
		If (gain_heavy > 0)
			Game.AdvanceSkill("Heavyarmor" , gain_heavy)
		EndIf	
	ElseIf Player.IsSprinting()
		If (gain_light > 0)
			Game.AdvanceSkill("Lightarmor" , gain_light*2)
		EndIf
		If (gain_heavy > 0)
			Game.AdvanceSkill("Heavyarmor" , gain_heavy*2)
		EndIf
	EndIf
EndEvent

Event OnObjectEquipped(Form akBaseObject, ObjectReference akReference)
	If ( akBaseObject as Armor != None )
		If ( (akBaseObject as Armor).GetSlotMask() == Armor.GetMaskForSlot(39) || Exceptions.Find(akBaseObject as Armor) >= 0 || CombatToggle.GetValueInt() == 1 )
			GoToState("StartEvaluation")
		ElseIf ( Player.IsInCombat() && slots.find(akBaseObject as Armor) < 0 )
			Player.UnequipItem(akBaseObject, abSilent = True )
			NoCombatChanges.Show()
		ElseIf ( !Player.IsInCombat() )
			GoToState("StartEvaluation")
		EndIf
	ElseIf ( akBaseObject as Weapon != None )
		GoToState("StartEvaluation")
	EndIf
EndEvent

Event OnObjectUnEquipped(Form akBaseObject, ObjectReference akReference)
	If ( akBaseObject as Armor != None )
		If ( (akBaseObject as Armor).GetSlotMask() == Armor.GetMaskForSlot(39) || Exceptions.Find(akBaseObject as Armor) >= 0 || CombatToggle.GetValueInt() == 1 || Player.GetItemCount(akBaseObject) == 0)
			GoToState("StartEvaluation")
		ElseIf ( Player.IsInCombat() && slots.find(akBaseObject as Armor) > 0 )
			Player.EquipItem(akBaseObject, abSilent = True )
			NoCombatChanges.Show()
		ElseIf ( !Player.IsInCombat() )
			GoToState("StartEvaluation")
		EndIf
	ElseIf ( akBaseObject as Weapon != None )
		GoToState("StartEvaluation")
	EndIf
EndEvent

State StartEvaluation

	Event OnObjectUnEquipped(Form akBaseObject, ObjectReference akReference)
	EndEvent
	
	Event OnObjectEquipped(Form akBaseObject, ObjectReference akReference)
	EndEvent

	Event OnUpdate()
		If !working 
			GoToState("EvaluateArmor")
		EndIf
	EndEvent
	
	Event OnBeginState()
		UnregisterForUpdate()
		RegisterForUpdate(0.5)
	EndEvent

EndState

State EvaluateArmor

	Event OnObjectUnEquipped(Form akBaseObject, ObjectReference akReference)
		GoToState("StartEvaluation")
	EndEvent
	
	Event OnObjectEquipped(Form akBaseObject, ObjectReference akReference)
		GoToState("StartEvaluation")
	EndEvent
	
	Event OnBeginState()
		working = True
		UnregisterForUpdate()
		Float valheavy = gain_heavy
		Float vallight = gain_light
		AdjustGains()
		If ( ArmorDebug.GetValue() == 1 )
			Debug.Messagebox("Light armor: "+vallight+" -> "+gain_light+"\nHeavy armor: "+valheavy+" -> "+gain_heavy)
		EndIf
		If ( GetState() == "EvaluateArmor" )
			RegisterForUpdate(UpdateInterval)
			GoToState("")
		EndIf
		working = False
	EndEvent
	
EndState

Function Adjustgains()
	Int i = 30
	Int type = -1
	Int pos = -1
	Float oldmass = mass
	weapon armed
	gain_heavy = 0
	gain_light = 0
	worn.Revert()
	mass = 0.0
	While (i < 62)
		slots[i - 30] = Player.GetWornForm(Armor.GetMaskForSlot(i)) as Armor
		worn.AddForm(slots[i - 30])
		pos = contributions.find(i)
		If ( pos >= 0 && slots[i - 30] != None )
			type = slots[i - 30].GetWeightClass()
			; Debug.MessageBox ("slot: "+i+"\narray-pos: "+pos+"\nitem: "+slots[i - 30].GetName()+"\ntype: "+type )
			If ( type == 1 )
				gain_heavy += Skillgain[pos]
			ElseIf ( type == 0 )
				gain_light += Skillgain[pos]
			ElseIf ( type == 2 )
				gain_light += Skillgain[pos] / 2
			EndIf
		EndIf
		i += 1
	EndWhile
	i = 0
	type = 10
	While (i < worn.Getsize())
		mass += worn.GetAt(i).GetWeight()
		i += 1
	EndWhile
	armed = player.GetEquippedWeapon(False)
	If (armed != None)
		mass += armed.GetWeight()
		type = player.GetEquippedItemType(1)
	EndIf
	if (type < 5 || type == 8 || armed == None)
		armed = player.GetEquippedWeapon(True)
		If (armed != None)
			mass += armed.GetWeight()
		Endif
	Endif
	mass /= 100.0
	player.RemoveSpell(MassEffect)
	player.modav("mass", mass-oldmass)
	player.AddSpell(MassEffect,False)
	player.modAV("CarryWeight",-0.1)
	Utility.Wait(0.1)
	player.modAV("CarryWeight",0.1)
EndFunction
